using System;
using Autofac;
using FitConnect.Encryption;
using FitConnect.Models;
using FitConnect.Models.v1.Api;
using FitConnect.Services.Models.v1.Submission;
using FluentAssertions;
using MockContainer;
using NUnit.Framework;
using SecurityEventToken = FitConnect.Models.SecurityEventToken;

namespace BasicUnitTest;

[TestFixture]
public class SecurityEventTokenTests {
    private const string rejectSubmission =
        SecurityEventToken.RejectSubmissionSchema;

    private FitEncryption _encryption = null!;

    private const string acceptSubmission =
        SecurityEventToken.AcceptSubmissionSchema;


    [SetUp]
    public void Setup() {
        var container = Container.Create();
        _encryption = new FitEncryption(container.Resolve<KeySet>(), null);
    }


    [Test]
    public void CreateJwt_AcceptSubmission() {
        var token = _encryption.CreateAcceptSecurityEventToken(new SubmissionForPickupDto {
            Id = Guid.NewGuid().ToString(), CaseId = Guid.NewGuid().ToString(),
            DestinationId = Guid.NewGuid().ToString()
        });
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(acceptSubmission);
        decoded.EventType.Should().Be(EventType.Accept);
    }

    [Test]
    public void CreateJwt_Reject_WithEncryptionIssue() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { Problems.EncryptionIssue }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithMissingSchema() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { Problems.MissingSchema }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);

        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithSchemaViolation() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { Problems.SchemaViolation }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithSyntaxViolation() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { Problems.SyntaxViolation }
        );
        Console.WriteLine(token);

        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithUnsupportedSchema() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { Problems.UnsupportedSchema }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_Reject_WithIncorrectAuthenticationTag() {
        var token = _encryption.CreateRejectSecurityEventToken(Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(),
            new[] { Problems.IncorrectAuthenticationTag }
        );
        Console.WriteLine(token);
        var decoded = new SecurityEventToken(token);
        decoded.Event?.Type.Should()
            .Be(rejectSubmission);
        decoded.EventType.Should().Be(EventType.Reject);
    }

    [Test]
    public void CreateJwt_WithEncryptionKey_ShouldWork() {
        _encryption.CreateSecurityEventToken(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(),
            Guid.NewGuid().ToString(), "DummyEvent", null);
    }

    [Test]
    public void CreateJwt_WithEncryptionKey_ShouldThrowAnException() {
        Assert.Throws<InvalidOperationException>(() =>
            new FitEncryption(_encryption.PrivateKeyDecryption, _encryption.PublicKeyEncryption,
                    _encryption.PublicKeyEncryption,
                    _encryption.PublicKeySignatureVerification, null)
                .CreateSecurityEventToken(Guid.NewGuid().ToString(), Guid.NewGuid().ToString(),
                    Guid.NewGuid().ToString(), "DummyEvent", null)
        );
    }
}
